@@ -1,11 +1,17 @@
 import logging
 import sys
+import warnings
 from datetime import datetime
 from logging import CRITICAL, DEBUG, ERROR, INFO, WARNING
 from pathlib import Path
 from sys import version_info
 from threading import Lock
-from typing import IO, Iterator, List, Optional, TYPE_CHECKING, Union
+from typing import IO, TYPE_CHECKING, Iterator, List, Optional, Union
+
+# noinspection PyProtectedMember
+from warnings import WarningMessage
+
+from streamlink.exceptions import StreamlinkWarning
 
 
 if TYPE_CHECKING:  # pragma: no cover
@@ -119,6 +125,59 @@ class StringFormatter(logging.Formatter):
         return super().format(record)
 
 
+class WarningLogRecord(logging.LogRecord):
+    msg: WarningMessage  # type: ignore[assignment]
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.name = "warnings"
+        self.levelname = self.msg.category.__name__ if self.msg.category else UserWarning.__name__
+        self.pathname = self.msg.filename
+        self._path = Path(self.pathname)
+        self.filename = self._path.name
+        self.module = self._path.stem
+        self.lineno = self.msg.lineno
+
+    def getMessage(self) -> str:
+        if self.msg.category and issubclass(self.msg.category, StreamlinkWarning):
+            return f"{self.msg.message}"
+        return f"{self.msg.message}\n  {self.pathname}:{self.lineno}"
+
+
+def _log_record_factory(name, level, fn, lno, msg, args, exc_info, func=None, sinfo=None, **kwargs):
+    if isinstance(msg, WarningMessage):
+        # noinspection PyTypeChecker
+        return WarningLogRecord(name, level, fn, lno, msg, args, exc_info, func, sinfo)
+
+    return _log_record_factory_default(name, level, fn, lno, msg, args, exc_info, func=None, sinfo=None, **kwargs)
+
+
+# borrowed from stdlib and modified, so that `WarningMessage` gets passed as `msg` to the `WarningLogRecord`
+def _showwarning(message, category, filename, lineno, file=None, line=None):
+    if file is not None:  # pragma: no cover
+        if _showwarning_default is not None:
+            # noinspection PyCallingNonCallable
+            _showwarning_default(message, category, filename, lineno, file, line)
+        return
+
+    warning = WarningMessage(message, category, filename, lineno, None, line)
+    kwargs = {"stacklevel": 2} if sys.version_info >= (3, 8) else {}
+    root.log(WARNING, warning, **kwargs)
+
+
+def capturewarnings(capture=False):
+    global _showwarning_default
+
+    if capture:
+        if _showwarning_default is None:
+            _showwarning_default = warnings.showwarning
+            warnings.showwarning = _showwarning
+    else:
+        if _showwarning_default is not None:
+            warnings.showwarning = _showwarning_default
+            _showwarning_default = None
+
+
 # noinspection PyShadowingBuiltins,PyPep8Naming
 def basicConfig(
     filename: Optional[Union[str, Path]] = None,
@@ -128,7 +187,8 @@ def basicConfig(
     format: str = FORMAT_BASE,
     style: str = FORMAT_STYLE,  # TODO: py38: Literal["%", "{", "$"]
     datefmt: str = FORMAT_DATE,
-    remove_base: Optional[List[str]] = None
+    remove_base: Optional[List[str]] = None,
+    capture_warnings: bool = False,
 ) -> logging.StreamHandler:
     with _config_lock:
         handler: logging.StreamHandler
@@ -142,7 +202,7 @@ def basicConfig(
             format,
             datefmt,
             style=style,
-            remove_base=remove_base or REMOVE_BASE
+            remove_base=remove_base or REMOVE_BASE,
         )
         handler.setFormatter(formatter)
 
@@ -150,9 +210,17 @@ def basicConfig(
         if level is not None:
             root.setLevel(level)
 
+        if capture_warnings:
+            capturewarnings(True)
+
     return handler
 
 
+_showwarning_default = None
+_log_record_factory_default = logging.getLogRecordFactory()
+logging.setLogRecordFactory(_log_record_factory)
+
+
 logging.setLoggerClass(StreamlinkLogger)
 root = logging.getLogger("streamlink")
 root.setLevel(WARNING)
@@ -162,10 +230,16 @@ levels = list(_levelToNames.values())
 
 __all__ = [
     "NONE",
+    "CRITICAL",
+    "ERROR",
+    "WARNING",
+    "INFO",
+    "DEBUG",
     "TRACE",
     "ALL",
     "StreamlinkLogger",
     "basicConfig",
     "root",
     "levels",
+    "capturewarnings",
 ]
