@@ -9,7 +9,7 @@ import logging
 import re
 
 from streamlink.buffers import RingBuffer
-from streamlink.plugin import Plugin, PluginArgument, PluginArguments, PluginError, pluginmatcher
+from streamlink.plugin import Plugin, PluginError, pluginargument, pluginmatcher
 from streamlink.plugin.api import validate
 from streamlink.plugin.api.websocket import WebsocketClient
 from streamlink.stream.stream import Stream, StreamIO
@@ -20,35 +20,33 @@ log = logging.getLogger(__name__)
 
 
 @pluginmatcher(re.compile(
-    r"https?://twitcasting\.tv/(?P<channel>[^/]+)"
+    r"https?://twitcasting\.tv/(?P<channel>[^/]+)",
 ))
+@pluginargument(
+    "password",
+    sensitive=True,
+    metavar="PASSWORD",
+    help="Password for private Twitcasting streams.",
+)
 class TwitCasting(Plugin):
-    arguments = PluginArguments(
-        PluginArgument(
-            "password",
-            sensitive=True,
-            metavar="PASSWORD",
-            help="Password for private Twitcasting streams."
-        )
-    )
     _STREAM_INFO_URL = "https://twitcasting.tv/streamserver.php?target={channel}&mode=client"
     _STREAM_REAL_URL = "{proto}://{host}/ws.app/stream/{movie_id}/fmp4/bd/1/1500?mode={mode}"
 
     _STREAM_INFO_SCHEMA = validate.Schema({
         validate.optional("movie"): {
             "id": int,
-            "live": bool
+            "live": bool,
         },
         validate.optional("fmp4"): {
             "host": str,
             "proto": str,
             "source": bool,
-            "mobilesource": bool
-        }
+            "mobilesource": bool,
+        },
     })
 
-    def __init__(self, url):
-        super().__init__(url)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.channel = self.match.group("channel")
 
     def _get_streams(self):
@@ -73,7 +71,7 @@ class TwitCasting(Plugin):
         else:
             mode = "base"  # Low quality
 
-        if (proto == '') or (host == '') or (not movie_id):
+        if (proto == "") or (host == "") or (not movie_id):
             raise PluginError(f"No stream available for user {self.channel}")
 
         real_stream_url = self._STREAM_REAL_URL.format(proto=proto, host=host, movie_id=movie_id, mode=mode)
@@ -127,7 +125,7 @@ class TwitCastingReader(StreamIO):
             self.buffer,
             stream.session,
             stream.url,
-            origin="https://twitcasting.tv/"
+            origin="https://twitcasting.tv/",
         )
 
     def open(self):
@@ -141,7 +139,7 @@ class TwitCastingReader(StreamIO):
         return self.buffer.read(
             size,
             block=self.wsclient.is_alive(),
-            timeout=self.timeout
+            timeout=self.timeout,
         )
 
 
