@@ -17,10 +17,14 @@ from streamlink.stream.stream import Stream
 from streamlink.utils.l10n import Language
 from streamlink.utils.parse import parse_xml
 
+
 log = logging.getLogger(__name__)
 
 
 class DASHStreamWriter(SegmentedStreamWriter):
+    reader: "DASHStreamReader"
+    stream: "DASHStream"
+
     @staticmethod
     def _get_segment_name(segment: Segment) -> str:
         return Path(urlparse(segment.url).path).resolve().name
@@ -70,8 +74,12 @@ class DASHStreamWriter(SegmentedStreamWriter):
 
 
 class DASHStreamWorker(SegmentedStreamWorker):
+    reader: "DASHStreamReader"
+    writer: "DASHStreamWriter"
+    stream: "DASHStream"
+
     def __init__(self, *args, **kwargs):
-        SegmentedStreamWorker.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.mpd = self.stream.mpd
         self.period = self.stream.period
 
@@ -156,8 +164,12 @@ class DASHStreamReader(SegmentedStreamReader):
     __worker__ = DASHStreamWorker
     __writer__ = DASHStreamWriter
 
-    def __init__(self, stream, representation_id, mime_type, *args, **kwargs):
-        SegmentedStreamReader.__init__(self, stream, *args, **kwargs)
+    worker: "DASHStreamWorker"
+    writer: "DASHStreamWriter"
+    stream: "DASHStream"
+
+    def __init__(self, stream: "DASHStream", representation_id, mime_type, *args, **kwargs):
+        super().__init__(stream, *args, **kwargs)
         self.mime_type = mime_type
         self.representation_id = representation_id
         log.debug("Opening DASH reader for: {0} ({1})".format(self.representation_id, self.mime_type))
@@ -177,7 +189,7 @@ class DASHStream(Stream):
         video_representation: Optional[Representation] = None,
         audio_representation: Optional[Representation] = None,
         period: float = 0,
-        **args
+        **args,
     ):
         """
         :param streamlink.Streamlink session: Streamlink session instance
@@ -222,7 +234,7 @@ class DASHStream(Stream):
         cls,
         session,
         url_or_manifest: str,
-        **args
+        **args,
     ) -> Dict[str, "DASHStream"]:
         """
         Parse a DASH manifest file and return its streams.
@@ -232,7 +244,7 @@ class DASHStream(Stream):
         :param args: Additional keyword arguments passed to :meth:`requests.Session.request`
         """
 
-        if url_or_manifest.startswith('<?xml'):
+        if url_or_manifest.startswith("<?xml"):
             mpd = MPD(parse_xml(url_or_manifest, ignore_ns=True))
         else:
             res = session.http.get(url_or_manifest, **session.http.valid_request_args(**args))
@@ -282,7 +294,7 @@ class DASHStream(Stream):
 
         log.debug("Available languages for DASH audio streams: {0} (using: {1})".format(
             ", ".join(available_languages) or "NONE",
-            lang or "n/a"
+            lang or "n/a",
         ))
 
         # if the language is given by the stream, filter out other languages that do not match
@@ -298,7 +310,7 @@ class DASHStream(Stream):
                 stream_name.append("{:0.0f}{}".format(vid.height or vid.bandwidth_rounded, "p" if vid.height else "k"))
             if audio and len(audio) > 1:
                 stream_name.append("a{:0.0f}k".format(aud.bandwidth))
-            ret.append(('+'.join(stream_name), stream))
+            ret.append(("+".join(stream_name), stream))
 
         # rename duplicate streams
         dict_value_list = defaultdict(list)
@@ -325,9 +337,9 @@ class DASHStream(Stream):
                 if n == 0:
                     ret_new[q] = items[n]
                 elif n == 1:
-                    ret_new[f'{q}_alt'] = items[n]
+                    ret_new[f"{q}_alt"] = items[n]
                 else:
-                    ret_new[f'{q}_alt{n}'] = items[n]
+                    ret_new[f"{q}_alt{n}"] = items[n]
         return ret_new
 
     def open(self):
